/*!
    \file weapon.h
    \brief a weapon that a  can use


    A weapon is characterized by:
     o firing-rate: the initial firing rate of a weapon (1/s = Herz)
     o slowdown-factor: this is a factor d: exp(-d*x), d is element of all positive R. it determines how fast the firing-rate will slow down. if no slowdown: d=0, the bigger d is, the faster the weapon will slow down!
     o energy-consumption: this determines the energy that has to be used to produce this projectile = costs per projectile

    Furthermore there are some other attributes, that will help to represent a firing
    weapon in this world:
     o sound file/ressource: this is a pointer to the sound-file/ressource. however it may be represented
     o shooting animation
*/


#ifndef _WEAPON_H
#define _WEAPON_H

#include "base_object.h"
#include "world_entity.h"

#define W_MAX_SLOTS 8
#define W_MAX_CONFIGS 4


// FORWARD DECLARATION
class Projectile;
class Weapon;
class Animation3D;
class TiXmlElement;


// typedef enum {
//   W_SHOOT,
//   W_EMPTY,
//   W_RELOAD,
//   W_SPECIAL1,
//   W_SPECIAL2,
//   W_SPECIAL3
// } WeaponSoundType;


//! An enumerator defining the States of a Weapon
typedef enum {
  W_NONE          =    0,    //!< No State at all (if set, there is something wrong, or the weapon is not yet availiable)
  W_SHOOT         =    1,    //!< The State of the Shooting
  W_RELOAD        =    2,    //!< The State of the Reloading
  W_ACTIVATING    =    3,    //!< The State in which the weapon gets activated
  W_DEACTIVATING  =    4,    //!< The State in which the weapon gets deactivated
  W_INACTIVE      =    5,    //!< The State where the weapon is inactive (unable to shoot)
  W_IDLE          =    6,    //!< The State where the weapon is idle

  W_STATES_COUNT  =    6     //!< This must match the count of the enumerations (without W_NONE)
} WeaponState;


//! a weapon can be left or right sided
#define    W_LEFT        0
#define    W_RIGHT       1

//! An abstract class, that describes weapons
/**
 * This is used as a container for all the different kinds of weapons that may exist
 * Animations/Sounds/etc. will be handled in the Extensions of this class.
 */
class Weapon : public WorldEntity
{
  friend class World;

 public:
  Weapon (PNode* parent, const Vector& coordinate, const Quaternion& direction);
  Weapon(const TiXmlElement* root);
  virtual ~Weapon ();

  void init();
  void loadParams(const TiXmlElement* root);

  void enable();
  void disable();
  bool isEnabled();

  void setProjectile(Projectile* projectile);
  Projectile* getProjectile();

  virtual void activate();
  virtual void deactivate();
  bool isActive();


  /** @param idle time in ms  */
  inline void setWeaponIdleTime(float idleTime) { this->idleTime = idleTime; };
  /** @returns idle time in ms */
  inline float getWeaponIdleTime() const { return this->idleTime; };
  /** @return true if idletime is elapsed else otherwise */
  inline bool hasWeaponIdleTimeElapsed() const { return (this->localTime>this->idleTime)?true:false; };

  /** @brief fires the weapon */
  virtual void fire() = 0;
  virtual void hit (WorldEntity* weapon, Vector* loc);
  virtual void destroy();

  virtual void tick(float time);
  virtual void weaponIdle();
  virtual void draw();

 protected:
  tList<WorldEntity>*  worldEntities;

  float                localTime;                 //<! this is the local time. important for shooting attributes like frequency
  float                idleTime;                  //<! the time a weapon needs before it can shoot again. eg. shooting frequency or actication/deactivateion delay
  float                slowDownFactor;            //<! if the shooting frequency is a linear function of time...

  SoundBuffer*         fireSound;
  SoundSource*         weaponSource;


 private:
  bool                 enabled;                    //<! states if the weapon is enabled or not
  Projectile*          projectile;          //<! the projectile used for this weapon
  //WeaponSound sound;
};

#endif /* _WEAPON_H */
